package eu.ruekov.ruakij.LinkedBeaconTeleporters;

import eu.ruekov.ruakij.LinkedBeaconTeleporters.customPlayerMoveEvent.CustomPlayerMoveEventHandler;
import eu.ruekov.ruakij.LinkedBeaconTeleporters.linkedBeaconTeleporter.LinkedBeaconTeleporterManager;
import eu.ruekov.ruakij.LinkedBeaconTeleporters.listener.OnBlockBreak;
import eu.ruekov.ruakij.LinkedBeaconTeleporters.listener.OnBlockPlace;
import eu.ruekov.ruakij.LinkedBeaconTeleporters.listener.OnCraftItemEvent;
import eu.ruekov.ruakij.LinkedBeaconTeleporters.listener.OnCustomPlayerMove;
import org.bukkit.Bukkit;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.plugin.Plugin;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.java.JavaPlugin;

import java.io.File;
import java.util.logging.Logger;

public class Main extends JavaPlugin {

    public static Plugin plugin;

    public static FileConfiguration config;
    public static FileConfiguration data;

    public static Logger log;

    public void onEnable() {
        log = getLogger();

        plugin = this;

        PluginManager pluginManager = Bukkit.getPluginManager();
        pluginManager.registerEvents(new OnBlockBreak(), this);
        pluginManager.registerEvents(new OnBlockPlace(), this);
        pluginManager.registerEvents(new OnCraftItemEvent(), this);

        loadConfigs();

        CustomPlayerMoveEventHandler cpmHandler = new CustomPlayerMoveEventHandler(this, config.getInt("locationCheck.interval"), true);
        cpmHandler.registerListener(new OnCustomPlayerMove());

        LinkedBeaconTeleporterManager.init();

        initAutoSave();

        log.info("Plugin activated");
    }

    public void onDisable() {
        saveData();

        log.info("Plugin deactivated");
    }

    public void loadConfigs() {
        // config.yml
        this.saveDefaultConfig();
        config = this.getConfig();

        // data.yml
        try{
            File dataFile = new File("plugins/"+ plugin.getName() +"/data.yml");
            dataFile.createNewFile();
            data = YamlConfiguration.loadConfiguration(
                    dataFile
            );
        }catch (Exception ex){
            log.severe("Could not load/create data.yml!");
            ex.printStackTrace();
            Bukkit.getPluginManager().disablePlugin(this);
        }
    }

    public static void saveData(){
        boolean lbtDataChanged = LinkedBeaconTeleporterManager.writeData();

        // Only run save, when data changed
        if(lbtDataChanged)
            saveFiles();
    }

    public static void saveFiles() {
        // data.yml
        try{
            File dataFile = new File("plugins/"+ plugin.getName() +"/data.yml");
            data.save(dataFile);
        }catch (Exception ex){
            log.severe("Could not save data.yml!");
            ex.printStackTrace();
        }
    }

    static void initAutoSave(){
        int autosaveInterval = config.getInt("config.interval")*20;
        Bukkit.getScheduler().scheduleSyncRepeatingTask(plugin, () -> {
            saveData();

        }, autosaveInterval, autosaveInterval);
    }
}
