package eu.ruekov.ruakij.LinkedBeaconTeleporters.customPlayerMoveEvent;

import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.entity.Player;
import org.bukkit.plugin.Plugin;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;

public class CustomPlayerMoveEventHandler {

    List<CustomPlayerMoveEventListener> listeners = new ArrayList<>();

    HashMap<UUID, Location> oldPlayerLoc = new HashMap<>();

    boolean locationChangeOnBlockChange;
    public CustomPlayerMoveEventHandler(Plugin plugin, int interval, boolean locationChangeOnBlockChange){
        this.locationChangeOnBlockChange = locationChangeOnBlockChange;

        startRunnable(plugin, interval);
    }
    public CustomPlayerMoveEventHandler(Plugin plugin){
        this(plugin, 5, false);
    }

    void startRunnable(Plugin plugin, int interval){

        Bukkit.getScheduler().scheduleSyncRepeatingTask(plugin, () -> {
            // Dont do anything without listeners
            if(listeners.size() == 0) return;

            // Check all players
            for(Player p : Bukkit.getOnlinePlayers()){
                UUID uuid = p.getUniqueId();
                Location loc = p.getLocation();

                // Get old-location
                Location oldLoc = oldPlayerLoc.get(uuid);

                // Create data-obj
                CustomPlayerMoveEvent e = new CustomPlayerMoveEvent(p, loc, oldLoc);

                // When player has an old-location
                if(oldLoc != null){
                    if(!locationChangeOnBlockChange){
                        double distance = loc.distance(oldLoc);
                        e.distance = distance;

                        if(distance > 0)
                            locationChangeListeners(e);
                    }
                    else if(loc.getBlockX() != oldLoc.getBlockX() ||
                            loc.getBlockY() != oldLoc.getBlockY() ||
                            loc.getBlockZ() != oldLoc.getBlockZ())
                        locationChangeListeners(e);
                }

                tickListeners(e);

                // Save old location
                oldPlayerLoc.put(uuid, loc);
            }

        }, interval, interval);
    }

    boolean blockLocationChanged(Location loc, Location oldLoc){
        return  loc.getBlockX() != oldLoc.getBlockX() ||
                loc.getBlockY() != oldLoc.getBlockY() ||
                loc.getBlockZ() != oldLoc.getBlockZ();
    }

    void tickListeners(CustomPlayerMoveEvent e){
        for(CustomPlayerMoveEventListener listener : listeners){
            listener.tick(e);
        }
    }
    void locationChangeListeners(CustomPlayerMoveEvent e){
        for(CustomPlayerMoveEventListener listener : listeners){
            listener.locationChange(e);
        }
    }


    public void registerListener(CustomPlayerMoveEventListener listener){
        listeners.add(listener);
    }
}
